
from zope.interface import implements, providedBy
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces import NotFound
from zope.security.checker import ProxyFactory
from zope.location.location import locate

from schooltool.rest.interfaces import ICommonEntry
from schooltool.rest.interfaces import ST_CONTEXT_IFC, ST_CONTEXT_ATTR


class TraverseViaGet(object):
    implements(IPublishTraverse)

    def publishTraverse(self, request, name):
        marker = object()
        value = self.get(name, marker)
        if (value is marker or
            value is None):
            raise NotFound(self, name, request)
        return value


class Entry(object):
    implements(ICommonEntry)

    __name__ = None
    __parent__ = None
    context = None

    def __init__(self, obj, parent=None):
        self.context = obj
        if parent is not None:
            locate(self, parent)
        self.__name__ = getattr(obj, '__name__', None)

    def __getattr__(self, name):
        for ifc in providedBy(self):
            if name not in ifc:
                continue
            adapter = ifc[name].queryTaggedValue(ST_CONTEXT_IFC)
            if adapter is None:
                continue
            attr = ifc[name].queryTaggedValue(ST_CONTEXT_ATTR)
            if attr is None:
                continue
            context = object.__getattribute__(self, attr)
            target = adapter(ProxyFactory(context))
            return getattr(target, name)
        raise AttributeError(self, name)

    def __setattr__(self, name, value):
        for ifc in providedBy(self):
            if name not in ifc:
                continue
            adapter = ifc[name].queryTaggedValue(ST_CONTEXT_IFC)
            if adapter is None:
                continue
            attr = ifc[name].queryTaggedValue(ST_CONTEXT_ATTR)
            if attr is None:
                continue
            context = object.__getattribute__(self, attr)
            target = adapter(ProxyFactory(context))
            setattr(target, name, value)
            return
        object.__setattr__(self, name, value)
