"""
SchoolTool REST API for persons interfaces.
"""

import zope.schema

from lazr.restful.declarations import collection_default_content
from lazr.restful.declarations import export_as_webservice_entry
from lazr.restful.declarations import export_as_webservice_collection
from lazr.restful.declarations import export_read_operation
from lazr.restful.declarations import exported
from lazr.restful.declarations import operation_parameters
from lazr.restful.declarations import operation_returns_collection_of

import schooltool.basicperson.interfaces
from schooltool.rest.interfaces import ITopLevelResource
from schooltool.rest.interfaces import ICommonEntry
from schooltool.rest.interfaces import export_interface


class IPerson(ICommonEntry):
    export_as_webservice_entry()

    title = exported(zope.schema.TextLine(title=u"Title", readonly=True))

    export_interface(schooltool.basicperson.interfaces.IBasicPerson,
                     ommit=('advisors', 'advisees'),
                     adapt='context')


class IPersons(ITopLevelResource):
    export_as_webservice_collection(IPerson)

    @collection_default_content()
    def allPersons():
        """Return all persons"""

    @operation_parameters(
        query=zope.schema.TextLine(
            title=u"First, last name or username.",
            required=True,
            ))
    @operation_returns_collection_of(IPerson)
    @export_read_operation()
    def search(query):
        """Search for a person"""
