"""
SchoolTool REST API for persons.
"""


from zope.interface import implements
from zope.intid.interfaces import IIntIds
from zope.component import getUtility

from schooltool import rest
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.catalog import buildQueryString
from schooltool.rest.person.interfaces import IPerson, IPersons
from schooltool.basicperson.person import getPersonCatalog


class Person(rest.common.Entry):
    implements(IPerson)

    @property
    def title(self):
        return self.context.title


class Persons(rest.common.TraverseViaGet,
              rest.webservice.TopLevelResource):
    implements(IPersons)

    __name__ = "persons"

    @property
    def container(self):
        app = ISchoolToolApplication(None)
        persons = app['persons']
        return persons

    def allPersons(self):
        persons = self.container
        result = []
        for person in persons.values():
            result.append(Person(person, self))
        return result

    def search(self, query):
        querystr = buildQueryString(query)
        if not querystr:
            return []
        catalog = getPersonCatalog()
        items = catalog['text'].apply(querystr)
        int_ids = getUtility(IIntIds)
        result = []
        for item_id in items:
            person = int_ids.queryObject(item_id)
            result.append(Person(person, self))
        return result

    def get(self, name, default=None):
        marker = object()
        result = self.container.get(name, marker)
        if result is marker:
            return default
        person = Person(result, self)
        return person

