"""
SchoolTool REST API.
"""

from lazr.restful import ServiceRootResource
from lazr.restful.interfaces import IWebServiceClientRequest
from lazr.restful.utils import get_current_web_service_request

from zope.interface import implements
from zope.component import adapts, getAdapters, queryMultiAdapter
from zope.location.location import Location
from zope.publisher.interfaces import NotFound
from zope.traversing.browser.interfaces import IAbsoluteURL

from schooltool.rest.interfaces import ISchoolToolAPI
from schooltool.rest.interfaces import ITopLevelResource
from schooltool.traverser.traverser import TraverserPlugin


class SchoolToolAPI(ServiceRootResource):
    implements(ISchoolToolAPI)

    def getTopLevelPublications(self):
        resources = dict(super(SchoolToolAPI, self).getTopLevelPublications())
        request = get_current_web_service_request()
        subscribers = getAdapters((self, request), ITopLevelResource)
        for name, adapter in subscribers:
            resources[name] = adapter
        return resources


class SchoolToolAPI_V10(SchoolToolAPI):
    pass


class TopLevelResource(Location):
    adapts(SchoolToolAPI, IWebServiceClientRequest)
    implements(ITopLevelResource)

    __name__ = None
    __parent__ = None
    request = None

    def __init__(self, api, request):
        self.__parent__ = api
        self.request = request
        Location.__init__(self)

    @property
    def context(self):
        return self.__parent__


class TopLevelResourceTraverserPlugin(TraverserPlugin):

    def traverse(self, name):
        resource = queryMultiAdapter((self.context, self.request),
                                     ITopLevelResource,
                                     name=name)
        if resource is None:
            raise NotFound(self.context, name, self.request)
        resource.__name__ = name
        return resource


class SchoolToolAPIAbsoluteURL(object):
    adapts(SchoolToolAPI, IWebServiceClientRequest)
    implements(IAbsoluteURL)

    context = None
    request = None

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __str__(self):
        return self.request.getWebServiceURL()

    def __call__(self):
        return str(self) + '/'
